/*!
 * \file
 * \brief 	Structures and functions for the EL weather
 * \ingroup	misc
 */
#ifndef __WEATHER_H__
#define __WEATHER_H__

/*!
 * \name    	Maximum number of thunders in one place
 */
/*! \{ */
#define MAX_THUNDERS 5
/*! \} */

/*!
 * \name	Rain related constants
 */
/*! \{ */
#define MAX_RAIN_DROPS 5000 /*!< The maximum number of rain drops in one place */
#define RAIN_SPEED 2        /*!< The speed of the rain drops */
#define rain_drop_len 5     /*!< The length of the rain drops */
/*! \} */


extern int seconds_till_rain_starts;    /*!< Seconds till the rain starts */
extern int seconds_till_rain_stops;     /*!< Seconts till the rain stops */
extern int is_raining;                  /*!< Specifies if it's raining - if it is, draw the raindrops */
extern int rain_sound;                  /*!< Specifies the rain sound */
extern int weather_light_offset;        /*!< Sets the current light offset */
extern int rain_light_offset;           /*!< Sets the current rain offset */
extern int thunder_light_offset;        /*!< Sets the current thunder light offset */
extern int lightning_text;              /*!< Sets the lightning texture */


/*!
 * The thunder structure
 */
typedef struct
{
	int in_use;
	int light_offset;
	int seconds_till_sound;
	int thunder_type;
	int time_since_started;
	int x;
	int y;
	int rot;
}thunder;

thunder thunders[MAX_THUNDERS];         /*!< The array containing the current thunders */

/*!
 * The rain drop structure - defines the start and end position
 */
typedef struct
{
	short x;
	short y;
	short x2;
	short y2;
}rain_drop;

rain_drop rain_drops[MAX_RAIN_DROPS];   /*!< Defines the number of rain drops */

/*!
 * \ingroup	other
 * \brief 	Builds the rain table.
 *
 * 		Builds the current rain table, called on init. Defines where the raindrops will go in the screen.
 *
 * \param   	None
 * \return  	None
 */
void build_rain_table();

/*!
 * \ingroup	weather_timer
 * \brief 	Updates the rain
 *
 * 		Updates the rain on the screen (moves the location a bit, it's just a 2d effect).
 *
 * \param   	None
 * \return  	None
 */
void update_rain();

/*!
 * \ingroup 	display_weather
 * \brief 	Renders the rain
 *
 *      	Draws the raindrops on the screen.
 *
 * \param   	None
 * \return  	None
 */
void render_rain();

/*!
 * \ingroup	weather_timer
 * \brief 	Check whether it needs to start raining
 *
 *      	The function checks every second if it should begin raining.
 *
 * \param   	None
 * \return  	None
 */
void rain_control();

/*!
 * \ingroup	weather_timer
 * \brief 	Controls the thunders
 *
 * 		Checks if it needs to start a thunder nearby.
 *
 * \param   	None
 * \return  	None
 */
void thunder_control();

/*!
 * \ingroup	network_misc
 * \brief 	add_thunder
 *
 *		Adds a thunder of the specified type with the given sound_delay.
 * 
 * \param   	type
 * \param   	sound_delay
 * \return  	None
 */
void add_thunder(int type,int sound_delay);

/*!
 * \ingroup	display_weather
 * \brief	Gets the current light level for the weather
 *
 * 		Gets the current light level for the weather.
 *
 * \param   	None
 * \return  	None
 */
void get_weather_light_level();

/*!
 * \ingroup	other
 * \brief 	Reset the thunders
 *
 *      	Resets the thunders.
 *
 * \param   	None
 * \return  	None
 */
void clear_thunders();
#endif

